package im.zego.live;

import android.app.Application;
import im.zego.live.callback.ZegoRoomCallback;
import im.zego.live.service.ZegoDeviceService;
import im.zego.live.service.ZegoFaceBeautifyService;
import im.zego.live.service.ZegoMessageService;
import im.zego.live.service.ZegoRoomService;
import im.zego.live.service.ZegoSoundEffectsService;
import im.zego.live.service.ZegoUserService;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.callback.IZegoEventHandler;
import im.zego.zegoexpress.constants.ZegoScenario;
import im.zego.zegoexpress.constants.ZegoStreamQualityLevel;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoEngineProfile;
import im.zego.zegoexpress.entity.ZegoStream;
import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMEventHandler;
import im.zego.zim.entity.ZIMError;
import im.zego.zim.entity.ZIMMessage;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;
import im.zego.zim.entity.ZIMUserInfo;
import im.zego.zim.enums.ZIMConnectionEvent;
import im.zego.zim.enums.ZIMConnectionState;
import im.zego.zim.enums.ZIMRoomEvent;
import im.zego.zim.enums.ZIMRoomState;
import java.util.ArrayList;
import java.util.HashMap;
import org.json.JSONObject;

/**
 * Zego Live 逻辑管理类
 * <p>
 * 描述：此类包含了 Zego Live 业务逻辑，管理不同模块的服务实例，并分发SDK交付过来的数据。
 */
public class ZegoRoomManager {

    private static volatile ZegoRoomManager singleton = null;

    private ZegoRoomManager() {
    }

    /**
     * 获取 ZegoRoomManager 唯一实例
     * <p>
     * 描述：这个方法用来获取 ZegoRoomManager 唯一实例。
     * <p>
     * 调用时机：任何时候
     *
     * @return ZegoRoomManager 唯一实例
     */
    public static ZegoRoomManager getInstance() {
        if (singleton == null) {
            synchronized (ZegoRoomManager.class) {
                if (singleton == null) {
                    singleton = new ZegoRoomManager();
                }
            }
        }
        return singleton;
    }

    private static final String TAG = "ZegoRoomManager";

    // 房间信息管理实例
    // 维护房间信息，房间状态和其他业务逻辑
    public ZegoRoomService roomService;

    // 用户信息管理实例
    // 维护在房间中的用户信息，登录了的用户信息和其他业务逻辑
    public ZegoUserService userService;

    // 信息管理实例，包含IM信息管理逻辑
    public ZegoMessageService messageService;

    // 美颜管理实例
    // 包含美颜的 启用/禁用 、美颜参数设置和脸型修整功能逻辑
    public ZegoFaceBeautifyService faceBeautifyService;

    // 音效管理实例
    // 包含音效业务逻辑
    public ZegoSoundEffectsService soundEffectService;

    // 设备管理实例
    // 包含视频采集、渲染、相关参数、播放流和其他业务逻辑
    public ZegoDeviceService deviceService;

    /**
     * 初始化SDK
     * <p>
     * 描述：该方法用来初始化 ZIM SDK 和 Express-audio SDK
     * <p>
     * 调用时机：在登录前调用，我们希望你在应用启动时调用。
     *
     * @param appID 项目ID，请在<a href="https://console.zego.im/dashboard">控制台</a>获取。
     * @param appSign 认证秘钥，请在<a href="https://console.zego.im/dashboard">控制台</a>获取。
     */
    public void init(long appID, String appSign, Application application) {
        roomService = new ZegoRoomService();
        userService = new ZegoUserService();
        messageService = new ZegoMessageService();
        deviceService = new ZegoDeviceService();

        ZegoEngineProfile profile = new ZegoEngineProfile();
        profile.appID = appID;
        profile.appSign = appSign;
        profile.scenario = ZegoScenario.COMMUNICATION;
        profile.application = application;
        ZegoExpressEngine engine = ZegoExpressEngine.createEngine(profile, new IZegoEventHandler() {
            @Override
            public void onNetworkQuality(String userID, ZegoStreamQualityLevel upstreamQuality,
                ZegoStreamQualityLevel downstreamQuality) {
                super.onNetworkQuality(userID, upstreamQuality, downstreamQuality);
            }

            @Override
            public void onCapturedSoundLevelUpdate(float soundLevel) {
                super.onCapturedSoundLevelUpdate(soundLevel);
            }


            @Override
            public void onRemoteSoundLevelUpdate(HashMap<String, Float> soundLevels) {
                super.onRemoteSoundLevelUpdate(soundLevels);
            }

            @Override
            public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList,
                JSONObject extendedData) {
                super.onRoomStreamUpdate(roomID, updateType, streamList, extendedData);
                if (roomService != null) {
                    roomService.onRoomStreamUpdate(roomID, updateType, streamList);
                }
            }
        });
        soundEffectService = new ZegoSoundEffectsService();
        faceBeautifyService = new ZegoFaceBeautifyService();
        faceBeautifyService.enableBeautify(true);

        ZegoZIMManager.getInstance().createZIM(appID, application);

        // 分配到特定的正在监听的service
        ZegoZIMManager.getInstance().zim.setEventHandler(new ZIMEventHandler() {
            @Override
            public void onConnectionStateChanged(ZIM zim, ZIMConnectionState state, ZIMConnectionEvent event,
                JSONObject extendedData) {
                super.onConnectionStateChanged(zim, state, event, extendedData);
                if (roomService != null) {
                    roomService.onConnectionStateChanged(zim, state, event, extendedData);
                }
            }

            @Override
            public void onError(ZIM zim, ZIMError errorInfo) {
                super.onError(zim, errorInfo);
            }

            @Override
            public void onTokenWillExpire(ZIM zim, int second) {
                super.onTokenWillExpire(zim, second);
                if (roomService != null) {
                    roomService.onRoomTokenWillExpire(second, roomService.roomInfo.getRoomID());
                }
            }

            @Override
            public void onReceivePeerMessage(ZIM zim, ArrayList<ZIMMessage> messageList, String fromUserID) {
                super.onReceivePeerMessage(zim, messageList, fromUserID);
                if (userService != null) {
                    userService.onReceivePeerMessage(zim, messageList, fromUserID);
                }
            }

            @Override
            public void onReceiveRoomMessage(ZIM zim, ArrayList<ZIMMessage> messageList, String fromRoomID) {
                super.onReceiveRoomMessage(zim, messageList, fromRoomID);
                if (messageService != null) {
                    messageService.onReceiveRoomMessage(zim, messageList, fromRoomID);
                }
            }

            @Override
            public void onRoomMemberJoined(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                super.onRoomMemberJoined(zim, memberList, roomID);
                if (userService != null) {
                    userService.onRoomMemberJoined(zim, memberList, roomID);
                }
            }

            @Override
            public void onRoomMemberLeft(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                super.onRoomMemberLeft(zim, memberList, roomID);
                if (userService != null) {
                    userService.onRoomMemberLeft(zim, memberList, roomID);
                }
            }

            @Override
            public void onRoomStateChanged(ZIM zim, ZIMRoomState state, ZIMRoomEvent event, JSONObject extendedData,
                String roomID) {
                super.onRoomStateChanged(zim, state, event, extendedData, roomID);
                if (roomService != null) {
                    roomService.onRoomStateChanged(zim, state, event, extendedData, roomID);
                }
            }

            @Override
            public void onRoomAttributesUpdated(ZIM zim, ZIMRoomAttributesUpdateInfo info, String roomID) {
                super.onRoomAttributesUpdated(zim, info, roomID);
                if (roomService != null) {
                    roomService.onRoomAttributesUpdated(zim, info, roomID);
                }
            }

            @Override
            public void onRoomAttributesBatchUpdated(ZIM zim, ArrayList<ZIMRoomAttributesUpdateInfo> infos,
                String roomID) {
                super.onRoomAttributesBatchUpdated(zim, infos, roomID);
            }
        });
    }

    /**
     * 这个方法用来反初始化SDK
     * <p>
     * 描述：这个方法用来反初始化SDK和释放占用资源
     * <p>
     * 调用时机：当SDK不再使用的时候。我们建议您在应用退出的时候调用此方法
     */
    public void unInit() {
        ZegoZIMManager.getInstance().destroyZIM();
        ZegoExpressEngine.destroyEngine(null);
    }

    /**
     * 上传本地日志到Zego服务器
     * <p>
     * 描述：当异常发生时，可以调用此方法上传本地日志到Zego服务器排查错误。
     * <p>
     * 调用时机：当发生异常时。
     *
     * @param callback 当日志上传成功或者失败的时候会触发该回调
     */
    public void uploadLog(final ZegoRoomCallback callback) {
        ZegoZIMManager.getInstance().zim
            .uploadLog(errorInfo -> callback.onRoomCallback(errorInfo.code.value()));
        ZegoExpressEngine.getEngine().uploadLog();
    }
}
