package im.zego.live.service;


import android.view.TextureView;

import im.zego.live.helper.UserInfoHelper;
import im.zego.live.helper.ZegoLiveHelper;
import im.zego.live.model.enums.ZegoAudioBitrate;
import im.zego.live.model.enums.ZegoDevicesType;
import im.zego.live.model.enums.ZegoVideoCode;
import im.zego.live.model.enums.ZegoVideoResolution;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.constants.ZegoOrientation;
import im.zego.zegoexpress.constants.ZegoVideoCodecID;
import im.zego.zegoexpress.constants.ZegoVideoConfigPreset;
import im.zego.zegoexpress.constants.ZegoViewMode;
import im.zego.zegoexpress.entity.ZegoAudioConfig;
import im.zego.zegoexpress.entity.ZegoCanvas;
import im.zego.zegoexpress.entity.ZegoVideoConfig;

/**
 * 设备管理类
 * <p>
 * 描述：这个类包含相关的设备设置逻辑，可以配置不同的设备设置。
 */
public class ZegoDeviceService {

    /**
     * 设置视频分辨率
     * <p>
     * 描述：该方法可以用来设置视频分辨率，更大的分辨率会消耗更多的网络带宽。
     * 可以根据服务需求和网络情况选择分辨率，默认为720P。
     * <p>
     * 调用时机：加入房间之后
     *
     * @param videoResolution 视频分辨率
     */
    public void setVideoResolution(ZegoVideoResolution videoResolution) {
        ZegoVideoConfigPreset configPreset = ZegoVideoConfigPreset.getZegoVideoConfigPreset(videoResolution.value());
        ZegoVideoConfig videoConfig = new ZegoVideoConfig(configPreset);
        ZegoExpressEngine.getEngine().setVideoConfig(videoConfig);
    }

    /**
     * 设置音频码率
     * <p>
     * 描述：这个方法用于设置音频码率，一个更大的音频码率会消耗更多的网络带宽。
     * 可以根据服务需求和网络情况选择音频码率，默认为48kbps。
     * <p>
     * 调用时机：加入房间之后
     *
     * @param audioBitrate 音频码率
     */
    public void setAudioBitrate(ZegoAudioBitrate audioBitrate) {
        ZegoAudioConfig audioConfig = new ZegoAudioConfig();
        audioConfig.bitrate = audioBitrate.value();
        ZegoExpressEngine.getEngine().setAudioConfig(audioConfig);
    }

    /**
     * 设置视频编解码方式
     * <p>
     * 描述：不同的设备支持不同的编解码格式。有一些设备不支持 H.265，我们推荐使用 H.264。
     * <p>
     * 调用时机：加入房间之后
     *
     * @param videoCodec 编解码类型。
     */
    public void setVideoCodec(ZegoVideoCode videoCodec) {
        ZegoVideoConfig videoConfig = ZegoExpressEngine.getEngine().getVideoConfig();
        if (videoCodec == ZegoVideoCode.H265) {
            videoConfig.setCodecID(ZegoVideoCodecID.H265);
        } else {
            videoConfig.setCodecID(ZegoVideoCodecID.DEFAULT);
        }
        ZegoExpressEngine.getEngine().setVideoConfig(videoConfig);
    }

    /**
     * 配置设备设置
     * <p>
     * 描述：根据业务需求配置设备设置
     * <p>
     * 调用时机：加入房间之后
     *
     * @param devicesType 需要配置的类型
     * @param enable      是否启用
     */
    public void setDeviceStatus(ZegoDevicesType devicesType, boolean enable) {
        switch (devicesType) {
            case LAYERED_CODING:
                if (enable) {
                    ZegoVideoConfig videoConfig = ZegoExpressEngine.getEngine().getVideoConfig();
                    videoConfig.setCodecID(ZegoVideoCodecID.SVC);
                    ZegoExpressEngine.getEngine().setVideoConfig(videoConfig);
                }
                break;
            case HARDWARE_ENCODER:
                ZegoExpressEngine.getEngine().enableHardwareEncoder(enable);
                break;
            case HARDWARE_DECODER:
                ZegoExpressEngine.getEngine().enableHardwareDecoder(enable);
                break;
            case NOISE_SUPPRESSION:
                ZegoExpressEngine.getEngine().enableANS(enable);
                ZegoExpressEngine.getEngine().enableTransientANS(enable);
                break;
            case ECHO_CANCELLATION:
                ZegoExpressEngine.getEngine().enableAEC(enable);
                break;
            case VOLUME_ADJUSTMENT:
                ZegoExpressEngine.getEngine().enableAGC(enable);
                break;
        }
    }

    /**
     * 相机相关操作
     * <p>
     * 描述：这个方法可以启用或停用相机
     * 当相机启用的时候，视频流会自动推送到远端用户
     * <p>
     * 调用时机：加入房间之后
     *
     * @param enable 是否启用相机 true: 启用 false: 停用
     */
    public void enableCamera(boolean enable) {
        ZegoExpressEngine.getEngine().enableCamera(enable);
    }

    /**
     * 麦克风相关操作
     * <p>
     * 描述：这个方法可以用于麦克风静音或者解除静音
     * 当麦克风开启的时候，音频流会自动推送到远端用户
     * <p>
     * 调用时机：加入房间之后
     *
     * @param mute 是否静音麦克风。true: 静音，false: 解除静音
     */
    public void muteMic(boolean mute) {
        ZegoExpressEngine.getEngine().muteMicrophone(mute);
    }

    /**
     * 切换前后置摄像头
     * <p>
     * 描述: 这个方法用于设置摄像头，SDK默认使用前置摄像头
     * <p>
     * 调用时机：加入房间之后
     *
     * @param isFront 是否使用前置摄像头。true: 使用前置摄像头 false: 使用后置摄像头
     */
    public void useFrontCamera(boolean isFront) {
        ZegoExpressEngine.getEngine().useFrontCamera(isFront);
    }

    /**
     * 拉视频流
     * <p>
     * 描述：可以用来直观地播放视频数据，默认播放音频数据
     * <p>
     * 调用时机：加入房间之后
     *
     * @param userID      想要播放的视频流的用户ID
     * @param textureView 想要渲染的目标View
     */
    public void playVideoStream(String userID, TextureView textureView) {
        ZegoCanvas zegoCanvas = new ZegoCanvas(textureView);
        zegoCanvas.viewMode = ZegoViewMode.ASPECT_FILL;

        if (UserInfoHelper.isUserIDSelf(userID)) {
            ZegoExpressEngine.getEngine().setAppOrientation(ZegoOrientation.ORIENTATION_0);
            ZegoExpressEngine.getEngine().startPreview(zegoCanvas);
        } else {
            String streamID = ZegoLiveHelper.getStreamID(userID);
            ZegoExpressEngine.getEngine().startPlayingStream(streamID, zegoCanvas);
        }
    }

    public void stopPlayStream(String userID) {
        if (UserInfoHelper.isUserIDSelf(userID)) {
            ZegoExpressEngine.getEngine().stopPreview();
        } else {
            String streamID = ZegoLiveHelper.getStreamID(userID);
            ZegoExpressEngine.getEngine().stopPlayingStream(streamID);
        }
    }
}
