package im.zego.live.service;

import android.text.TextUtils;
import android.util.Log;
import com.google.gson.Gson;
import im.zego.live.ZegoRoomManager;
import im.zego.live.ZegoZIMManager;
import im.zego.live.callback.ZegoRoomCallback;
import im.zego.live.constants.ZegoRoomConstants;
import im.zego.live.helper.UserInfoHelper;
import im.zego.live.helper.ZegoRoomAttributesHelper;
import im.zego.live.listener.ZegoRoomServiceListener;
import im.zego.live.model.OperationAction;
import im.zego.live.model.OperationActionType;
import im.zego.live.model.OperationCommand;
import im.zego.live.model.ZegoCoHostSeatModel;
import im.zego.live.model.ZegoRoomInfo;
import im.zego.live.model.ZegoRoomUserRole;
import im.zego.live.model.ZegoUserInfo;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoRoomConfig;
import im.zego.zegoexpress.entity.ZegoStream;
import im.zego.zegoexpress.entity.ZegoUser;
import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMTokenRenewedCallback;
import im.zego.zim.entity.ZIMError;
import im.zego.zim.entity.ZIMRoomAdvancedConfig;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;
import im.zego.zim.entity.ZIMRoomInfo;
import im.zego.zim.enums.ZIMConnectionEvent;
import im.zego.zim.enums.ZIMConnectionState;
import im.zego.zim.enums.ZIMErrorCode;
import im.zego.zim.enums.ZIMRoomAttributesUpdateAction;
import im.zego.zim.enums.ZIMRoomEvent;
import im.zego.zim.enums.ZIMRoomState;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;

/**
 * ZegoLive 信息管理
 * <p>
 * 描述：此类包含房间信息管理逻辑，例如创建房间、加入房间、离开房间、禁止房间文字聊天等等
 */
public class ZegoRoomService {

    // 房间状态监听
    private ZegoRoomServiceListener listener;

    // 房间信息，在加入房间之后会被赋值，在房间状态更新的时候会同步更新
    public ZegoRoomInfo roomInfo = new ZegoRoomInfo();

    public OperationCommand operation = new OperationCommand();

    private static final String TAG = "ZegoRoomService";

    /**
     * 创建房间
     * <p>
     * 描述：这个方法可以用来创建房间。当房间创建成功，房间创建者默认为房主
     * <p>
     * 调用时机：用户登录之后
     *
     * @param roomID   房间ID，房间唯一标识，不允许为空。
     * @param roomName 房间名字，用于房间显示名字，不允许为空。
     * @param callback 创建房间回调。
     */
    public void createRoom(String roomID, String roomName, final ZegoRoomCallback callback) {
        ZegoUserInfo localUserInfo = ZegoRoomManager.getInstance().userService.localUserInfo;
        localUserInfo.setRole(ZegoRoomUserRole.Host);

        roomInfo.setRoomID(roomID);
        roomInfo.setRoomName(roomName);
        roomInfo.setHostID(localUserInfo.getUserID());

        ZIMRoomInfo zimRoomInfo = new ZIMRoomInfo();
        zimRoomInfo.roomID = roomID;
        zimRoomInfo.roomName = roomName;

        HashMap<String, String> roomAttributes = new HashMap<>();
        roomAttributes.put(ZegoRoomConstants.KEY_ROOM_INFO, ZegoRoomAttributesHelper.gson.toJson(roomInfo));
        ZIMRoomAdvancedConfig config = new ZIMRoomAdvancedConfig();
        config.roomAttributes = roomAttributes;

        ZegoZIMManager.getInstance().zim.createRoom(zimRoomInfo, config, (roomInfo, errorInfo) -> {
            if (errorInfo.code == ZIMErrorCode.SUCCESS) {
                loginRTCRoom(roomID, localUserInfo);
            }
            if (callback != null) {
                callback.onRoomCallback(errorInfo.code.value());
            }
        });
    }

    /**
     * 加入房间
     * <p>
     * 描述：这个方法用于加入房间，这个房间必须是已经存在的房间。
     * <p>
     * 调用时机：用户登录之后
     *
     * @param roomID   想要登录的房间ID，不允许为空。
     * @param callback 进入房间回调
     */
    public void joinRoom(String roomID, final ZegoRoomCallback callback) {
        ZegoUserInfo localUserInfo = ZegoRoomManager.getInstance().userService.localUserInfo;
        localUserInfo.setRole(ZegoRoomUserRole.Participant);

        ZegoZIMManager.getInstance().zim.joinRoom(roomID, (roomInfo, errorInfo) -> {
            if (errorInfo.code == ZIMErrorCode.SUCCESS) {
                loginRTCRoom(roomID, localUserInfo);
                this.roomInfo.setRoomID(roomInfo.baseInfo.roomID);
                this.roomInfo.setRoomName(roomInfo.baseInfo.roomName);
                ZegoZIMManager.getInstance().zim.queryRoomAllAttributes(roomID, (roomID1, roomAttributes, errorInfo1) -> {
                        ZIMRoomAttributesUpdateInfo updateInfo = new ZIMRoomAttributesUpdateInfo();
                        updateInfo.roomAttributes = roomAttributes;
                        updateInfo.action = ZIMRoomAttributesUpdateAction.SET;
                        onRoomAttributesUpdated(ZegoZIMManager.getInstance().zim,updateInfo,roomID);

                        if (callback != null) {
                            callback.onRoomCallback(errorInfo.code.value());
                        }
                    });
            } else {
                if (callback != null) {
                    callback.onRoomCallback(errorInfo.code.value());
                }
            }
        });
    }

    private void loginRTCRoom(String roomID, ZegoUserInfo localUserInfo) {
        ZegoUser user = new ZegoUser(localUserInfo.getUserID(), localUserInfo.getUserName());
        ZegoRoomConfig roomConfig = new ZegoRoomConfig();
        ZegoExpressEngine.getEngine().loginRoom(roomID, user, roomConfig);
        ZegoExpressEngine.getEngine().startSoundLevelMonitor(1000);
    }

    /**
     * 离开房间
     * <p>
     * 描述：这个方法用于离开房间，当房主退出房间时房间将会解散，所有用户都会强行退出房间。
     * <p>
     * 方法调用时机：在加入房间之后
     *
     * @param callback 离开房间回调
     */
    public void leaveRoom(final ZegoRoomCallback callback) {
        if (UserInfoHelper.isSelfInRequestedCoHost()) {
            ZegoRoomManager.getInstance().userService.cancelRequestToCoHost(errorCode -> {
            });
        }
        if (UserInfoHelper.isSelfCoHost()) {
            ZegoRoomManager.getInstance().userService.leaveSeat(null, errorCode -> {
            });
        }

        ZegoMessageService messageService = ZegoRoomManager.getInstance().messageService;
        if (messageService != null) {
            messageService.reset();
        }
        ZegoUserService userService = ZegoRoomManager.getInstance().userService;
        if (userService != null) {
            userService.leaveRoom();
        }
        reset();

        ZegoExpressEngine.getEngine().stopSoundLevelMonitor();
        ZegoExpressEngine.getEngine().stopPublishingStream();

        ZegoExpressEngine.getEngine().logoutRoom(roomInfo.getRoomID());

        ZegoZIMManager.getInstance().zim.leaveRoom(roomInfo.getRoomID(), (roomID, errorInfo) -> {
            Log.d(TAG, "leaveRoom() called with: errorInfo = [" + errorInfo.code + "]" + errorInfo.message);
            if (callback != null) {
                callback.onRoomCallback(errorInfo.code.value());
            }
        });
    }

    /**
     * 刷新Token
     * <p>
     * 描述：在开发者收到 [onRoomTokenWillExpire] 回调之后，可以使用此API更新token确保后续RTC&ZIM方法可用。
     *
     * @param token  需要刷新的token
     * @param roomID 房间ID
     */
    public void renewToken(String token, String roomID) {
        ZegoZIMManager.getInstance().zim.renewToken(token, new ZIMTokenRenewedCallback() {
            @Override
            public void onTokenRenewed(String token, ZIMError errorInfo) {

            }
        });
    }

    void reset() {
        roomInfo.setRoomName("");
        roomInfo.setHostID("");
        operation = new OperationCommand();
    }

    public void setListener(ZegoRoomServiceListener listener) {
        this.listener = listener;
    }

    public void onRoomStateChanged(ZIM zim, ZIMRoomState state, ZIMRoomEvent event, JSONObject extendedData,
                                   String roomID) {
        Log.d(TAG, "onRoomStateChanged() called with: zim = [" + zim + "], state = [" + state + "], event = [" + event
                + "], extendedData = [" + extendedData + "], roomID = [" + roomID + "]");
        if (state == ZIMRoomState.CONNECTED) {
            boolean newInRoom = StringUtils.isEmpty(this.roomInfo.getHostID());
            if (!newInRoom && !TextUtils.isEmpty(roomID)) {
                ZegoZIMManager.getInstance().zim.queryRoomAllAttributes(roomID, (roomID1, roomAttributes, errorInfo) -> {
                        boolean hostLeft = errorInfo.getCode() == ZIMErrorCode.SUCCESS
                            && !roomAttributes.keySet().contains(ZegoRoomConstants.KEY_ROOM_INFO);
                        boolean roomNotExisted = errorInfo.getCode() == ZIMErrorCode.ROOM_DOES_NOT_EXIST;
                        if (hostLeft || roomNotExisted) {
                            if (listener != null) {
                                listener.onReceiveRoomInfoUpdate(null);
                            }
                        } else {
                            String userID = ZegoRoomManager.getInstance().userService.localUserInfo.getUserID();
                            ZegoRoomListService.heartBeat(userID, roomID1, false, null);
                        }
                    });
            }
        } else if (state == ZIMRoomState.DISCONNECTED) {
            if (listener != null) {
                listener.onReceiveRoomInfoUpdate(null);
            }
        }
    }

    public void onRoomAttributesUpdated(ZIM zim, ZIMRoomAttributesUpdateInfo info, String roomID) {
        Log.d(TAG,
                "onRoomAttributesUpdated() called with: info.action = [" + info.action + "], info.roomAttributes = ["
                        + Collections.singletonList(info.roomAttributes) + "], roomID = [" + roomID
                        + "]");
        if (info.action == ZIMRoomAttributesUpdateAction.DELETE) {
            String roomJson = info.roomAttributes.get(ZegoRoomConstants.KEY_ROOM_INFO);
            if (TextUtils.isEmpty(roomJson)) {
                if (listener != null) {
                    listener.onReceiveRoomInfoUpdate(null);
                }
                return;
            }
        }

        Gson gson = ZegoRoomAttributesHelper.gson;
        String roomJson = info.roomAttributes.get(ZegoRoomConstants.KEY_ROOM_INFO);
        if (StringUtils.isNotEmpty(roomJson)) {
            ZegoRoomInfo roomInfo = gson.fromJson(roomJson, ZegoRoomInfo.class);
            this.roomInfo = roomInfo;
            if (listener != null) {
                listener.onReceiveRoomInfoUpdate(roomInfo);
            }
        }

        String actionJson = info.roomAttributes.get(ZegoRoomConstants.KEY_ACTION);
        if (StringUtils.isNotEmpty(actionJson)) {
            OperationAction action = gson.fromJson(actionJson, OperationAction.class);
            // 如果 seq 不可用
            // 只有房主可以重新发送房间参数
            if (!operation.isSeqValid(action.getSeq())) {
                if (UserInfoHelper.isSelfHost()) {
                    resendRoomAttributes(info.roomAttributes, action);
                }
                return;
            } else {
                operation.setAction(action);
            }

            // 更新麦位列表 & requestCoHostList
            operation.update(info.roomAttributes);

            ZegoRoomManager.getInstance().userService.coHostList = operation.getSeatList();

            List<ZegoUserInfo> userInfoList = ZegoRoomManager.getInstance().userService.getUserList();
            for (ZegoUserInfo zegoUserInfo : userInfoList) {
                zegoUserInfo.setHasRequestedCoHost(operation.getRequestCoHostList().contains(zegoUserInfo.getUserID()));

                if (UserInfoHelper.isUserIDCoHost(zegoUserInfo.getUserID())) {
                    zegoUserInfo.setHasInvited(false);
                }
            }

            // 如果连麦者不在用户列表，这个成员可能断线了或者离开房间了
            // 应该让他下麦，确保只有房主能够执行此操作
            if (UserInfoHelper.isSelfHost()) {
                List<ZegoCoHostSeatModel> coHostList = ZegoRoomManager.getInstance().userService.coHostList;
                for (ZegoCoHostSeatModel model : coHostList) {
                    boolean isFound = false;
                    for (ZegoUserInfo zegoUserInfo : userInfoList) {
                        if (Objects.equals(model.getUserID(), zegoUserInfo.getUserID()) && StringUtils.isNotEmpty(zegoUserInfo.getUserID())) {
                            isFound = true;
                            break;
                        }
                    }

                    if (!isFound) {
                        ZegoRoomManager.getInstance().userService.leaveSeat(model.getUserID(), errorCode -> {

                        });
                    }
                }
            }

            if (!UserInfoHelper.isSelfCoHost()) {
                ZegoExpressEngine.getEngine().stopPublishingStream();
            }

            switch (operation.getAction().getType()) {
                case Mic:
                case Camera:
                case Mute:
                case TakeSeat:
                    if (listener != null) {
                        listener.onReceiveCoHostListUpdate(operation.getAction());
                    }
                    break;
                case LeaveSeat:
                    if (!UserInfoHelper.isSelfCoHost()) {
                        ZegoExpressEngine.getEngine().stopPublishingStream();
                    }
                    if (listener != null) {
                        listener.onReceiveCoHostListUpdate(operation.getAction());
                    }
                    break;
            }

            ZegoUserService userService = ZegoRoomManager.getInstance().userService;
            if (userService != null) {
                userService.onRoomAttributesUpdated(info.roomAttributes, operation);
            }
        }
    }

    private void resendRoomAttributes(HashMap<String, String> roomAttributes, OperationAction action) {
        String roomID = this.roomInfo.getRoomID();

        OperationCommand operation = this.operation.copy();
        operation.setAction(action);
        operation.addSeq(action.getSeq());

        operation.updateForResend(roomAttributes);

        HashMap<String, String> map;
        if (operation.getAction().getType() == OperationActionType.Mic
                || operation.getAction().getType() == OperationActionType.Camera
                || operation.getAction().getType() == OperationActionType.Mute
                || operation.getAction().getType() == OperationActionType.TakeSeat
                || operation.getAction().getType() == OperationActionType.LeaveSeat
        ) {
            map = operation.getAttributes(OperationCommand.OperationAttributeTypeSeat);
        } else {
            map = operation.getAttributes(OperationCommand.OperationAttributeTypeRequestCoHost);
        }

        Log.d(TAG, "resendRoomAttributes() called with: roomAttributes = [" + Collections.singletonList(map) + "]");

        ZegoRoomAttributesHelper.setRoomAttributes(map, roomID, ZegoRoomAttributesHelper.getAttributesSetConfig(), errorCode -> {
        });
    }

    public void onConnectionStateChanged(ZIM zim, ZIMConnectionState state, ZIMConnectionEvent event,
                                         JSONObject extendedData) {
        if (listener != null) {
            listener.onConnectionStateChanged(state, event);
        }
    }

    public void onRoomTokenWillExpire(int remainTimeInSecond, String roomID) {
        if (listener != null) {
            listener.onRoomTokenWillExpire(remainTimeInSecond, roomID);
        }
    }

    public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, List<ZegoStream> streamList) {
        for (ZegoStream zegoStream : streamList) {
            if (updateType == ZegoUpdateType.DELETE) {
                ZegoDeviceService deviceService = ZegoRoomManager.getInstance().deviceService;
                deviceService.stopPlayStream(zegoStream.user.userID);
            }
        }
    }
}