package im.zego.live.service;

import java.util.Objects;

import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.ZegoMediaPlayer;
import im.zego.zegoexpress.constants.ZegoMediaPlayerState;
import im.zego.zegoexpress.constants.ZegoReverbPreset;
import im.zego.zegoexpress.constants.ZegoVoiceChangerPreset;

/**
 * 音效类
 * <p>
 * 描述：这个类包含了音效逻辑
 */
public class ZegoSoundEffectsService {
    public static final int DEFAULT_MUSIC_VOLUME = 50;
    public static final int DEFAULT_VOICE_VOLUME = 50;

    private final ZegoMediaPlayer mediaPlayer;
    private String currentBgmPath;

    public ZegoSoundEffectsService() {
        mediaPlayer = ZegoExpressEngine.getEngine().createMediaPlayer();
        mediaPlayer.enableAux(true);
        mediaPlayer.enableRepeat(true);
    }

    public void reset() {
        currentBgmPath = "";
        setBGMVolume(DEFAULT_MUSIC_VOLUME);
        setVoiceVolume(DEFAULT_VOICE_VOLUME);
        ZegoExpressEngine.getEngine().setReverbPreset(ZegoReverbPreset.NONE);
        ZegoExpressEngine.getEngine().setVoiceChangerPreset(ZegoVoiceChangerPreset.NONE);
        stopBGM();
    }

    /**
     * 加载背景音乐文件
     * <p>
     * 描述：这个方法可以通过设置文件路径的方式播放音乐
     * <p>
     * 调用时机：加入房间之后
     *
     * @param path 音乐资源路径
     */
    public void loadBGM(String path) {
        if (mediaPlayer.getCurrentState() == ZegoMediaPlayerState.PLAYING) {
            if (!Objects.equals(currentBgmPath, path)) {
                stopBGM();
                loadResource(path);
            }
        } else {
            loadResource(path);
        }
    }

    private void loadResource(String path) {
        mediaPlayer.loadResource(path, errorCode -> {
            if (errorCode == 0) {
                currentBgmPath = path;
                startBGM();
            }
        });
    }

    /**
     * 开始播放背景音乐
     * <p>
     * 描述：这个方法可以用于在调用 stopBGM 方法之后重新播放音乐
     * <p>
     * 调用时机：加入房间并且调用loadBGM之后
     */
    public void startBGM() {
        if (mediaPlayer.getCurrentState() != ZegoMediaPlayerState.PLAYING) {
            mediaPlayer.start();
        }
    }

    /**
     * 停止播放背景音乐
     * <p>
     * 描述：这个方法可以用于停止播放背景音乐，可以通过调用 startBGM 重新播放音乐
     * <p>
     * 调用时机：进入房间并且调用loadBGM方法之后
     */
    public void stopBGM() {
        if (mediaPlayer.getCurrentState() == ZegoMediaPlayerState.PLAYING) {
            mediaPlayer.stop();
        }
    }

    /**
     * 设置背景音乐音量
     * <p>
     * 描述：音乐音量范围为[0, 200]。默认为60。
     * <p>
     * 调用时机：进入房间之后
     *
     * @param volume 音乐音量
     */
    public void setBGMVolume(int volume) {
        mediaPlayer.setVolume(volume);
    }

    /**
     * 设置采集音量
     * <p>
     * 描述：音量范围为[0, 100]。默认为50
     * <p>
     * 调用时机：进入房间之后
     *
     * @param volume 采集音量
     */
    public void setVoiceVolume(int volume) {
        ZegoExpressEngine.getEngine().setCaptureVolume(volume * 2);
    }

    /**
     * 设置声音变化
     * <p>
     * 描述：这个方法通过声音效果预设来改变声音
     * <p>
     * 调用时机：进入房间之后
     *
     * @param voicePreset 需要改变的声音类型
     */
    public void setVoiceChangeType(ZegoVoiceChangerPreset voicePreset) {
        ZegoExpressEngine.getEngine().setVoiceChangerPreset(voicePreset);
    }

    /**
     * 设置混响
     * <p>
     * 描述：这个方法可以用来设置房间混响效果
     * <p>
     * 调用时机：加入房间之后
     *
     * @param reverbPreset 混响类型
     */
    public void setReverbPreset(ZegoReverbPreset reverbPreset) {
        ZegoExpressEngine.getEngine().setReverbPreset(reverbPreset);
    }
}