package im.zego.live.service;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import im.zego.live.ZegoRoomManager;
import im.zego.live.ZegoZIMManager;
import im.zego.live.callback.ZegoOnlineRoomUserListCallback;
import im.zego.live.callback.ZegoOnlineRoomUsersNumCallback;
import im.zego.live.callback.ZegoRoomCallback;
import im.zego.live.constants.ZegoRoomErrorCode;
import im.zego.live.helper.UserInfoHelper;
import im.zego.live.helper.ZegoLiveHelper;
import im.zego.live.helper.ZegoRoomAttributesHelper;
import im.zego.live.listener.ZegoUserServiceListener;
import im.zego.live.model.OperationAction;
import im.zego.live.model.OperationActionType;
import im.zego.live.model.OperationCommand;
import im.zego.live.model.ZegoCoHostSeatModel;
import im.zego.live.model.ZegoCustomCommand;
import im.zego.live.model.ZegoRoomInfo;
import im.zego.live.model.ZegoRoomUserRole;
import im.zego.live.model.ZegoUserInfo;
import im.zego.live.util.Triple;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMMessageSentCallback;
import im.zego.zim.callback.ZIMRoomMemberQueriedCallback;
import im.zego.zim.entity.ZIMCommandMessage;
import im.zego.zim.entity.ZIMError;
import im.zego.zim.entity.ZIMMessage;
import im.zego.zim.entity.ZIMMessageSendConfig;
import im.zego.zim.entity.ZIMRoomAttributesSetConfig;
import im.zego.zim.entity.ZIMRoomMemberQueryConfig;
import im.zego.zim.entity.ZIMUserInfo;
import im.zego.zim.enums.ZIMErrorCode;
import im.zego.zim.enums.ZIMMessageType;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 用户信息管理类
 * <p>描述：这个类包含了用户信息管理逻辑
 * 例如登录、登出、获取已登录的用户信息、获取房间中用户列表和添加连麦等等
 */
public class ZegoUserService {
    private static final String TAG = "ZegoUserService";
    private static final long RESET_INVITED_DELAY_TIME = 60 * 1000L;

    // 用户状态相关
    private ZegoUserServiceListener listener;

    // 本地已登录用户信息
    public ZegoUserInfo localUserInfo;
    // 房间内用户列表，可以用来显示房间的用户列表
    private final List<ZegoUserInfo> userList = new ArrayList<>();
    // 连麦列表，可以用来显示房间的连麦用户列表
    public List<ZegoCoHostSeatModel> coHostList = new ArrayList<>();

    private final Handler handler = new Handler(Looper.getMainLooper());
    private final Map<String, ZegoUserInfo> userMap = new HashMap<>();

    /**
     * 登录
     * <p>
     * 描述：传入用户ID和用户名来登录到ZEGO Live服务器
     * <p>
     * 调用时机：在SDK初始化之后
     *
     * @param userInfo 用户信息，可以用来传入用户ID和用户名
     * @param token    鉴权Token，获取Token可以参考文档：https://doc-zh.zego.im/article/11611
     * @param callback 登录回调
     */
    public void login(ZegoUserInfo userInfo, String token, final ZegoRoomCallback callback) {
        ZIMUserInfo zimUserInfo = new ZIMUserInfo();
        zimUserInfo.userID = userInfo.getUserID();
        zimUserInfo.userName = userInfo.getUserName();
        ZegoZIMManager.getInstance().zim.login(zimUserInfo, token, errorInfo -> {
            Log.d(TAG, "onLoggedIn() called with: errorInfo = [" + errorInfo.code + ", "
                    + errorInfo.message + "]");
            if (errorInfo.code == ZIMErrorCode.SUCCESS) {
                localUserInfo = new ZegoUserInfo();
                localUserInfo.setUserID(userInfo.getUserID());
                localUserInfo.setUserName(userInfo.getUserName());
            }
            if (callback != null) {
                callback.onRoomCallback(errorInfo.code.value());
            }
        });
    }

    /**
     * 用户登出
     * <p>
     * 描述：这个方法用来登出最近登录的用户
     * <p>
     * 调用时机：在用户登录之后
     */
    public void logout() {
        Log.d(TAG, "logout() called");
        ZegoZIMManager.getInstance().zim.logout();
        leaveRoom();
    }

    void leaveRoom() {
        userList.clear();
        userMap.clear();
        coHostList.clear();
    }

    /**
     * 获取房间内的用户列表
     * <p>
     * 描述：调用这个方法获取房间内用户列表
     * <p>
     * 调用时机：在加入房间之后
     *
     * @param nextFlag 传入null将会获取100个最近加入房间的用户
     *                 传入nextFlag值返回上次查询检索用户自最后一次查询的列表。
     * @param callback 获取用户列表回调
     */
    public void getOnlineRoomUsers(String nextFlag, ZegoOnlineRoomUserListCallback callback) {
        ZegoRoomInfo roomInfo = ZegoRoomManager.getInstance().roomService.roomInfo;
        ZIMRoomMemberQueryConfig config = new ZIMRoomMemberQueryConfig();
        config.count = 1000;
        config.nextFlag = nextFlag;
        ZegoZIMManager.getInstance().zim.queryRoomMemberList(roomInfo.getRoomID(), config, new ZIMRoomMemberQueriedCallback() {
            @Override
            public void onRoomMemberQueried(String roomID, ArrayList<ZIMUserInfo> memberList, String nextFlag,
                ZIMError errorInfo) {
                if (callback != null) {
                    List<ZegoUserInfo> userList = generateRoomUsers(memberList);
                    callback.onUserListCallback(errorInfo.code.value(), userList, nextFlag);
                }
            }
        });
    }

    /**
     * 获取房间内用户人数
     * <p>
     * 描述：调用这个方法可以获取房间内用户人数
     * <p>
     * 调用时机：在加入房间之后
     *
     * @param callback 获取房间用户回调
     */
    public void getOnlineRoomUsersNum(ZegoOnlineRoomUsersNumCallback callback) {
        ZegoRoomInfo roomInfo = ZegoRoomManager.getInstance().roomService.roomInfo;
        ZegoZIMManager.getInstance().zim.queryRoomOnlineMemberCount(roomInfo.getRoomID(),
            (roomID, count, errorInfo) -> {
                if (callback != null) {
                    callback.onUserCountCallback(errorInfo.code.value(), count);
                }
            });
    }

    /**
     * 添加连麦
     * <p>
     * 描述：调用这个方法可以邀请一个在房间的用户参与连麦
     * <p>
     * 调用时机：在加入房间之后
     *
     * @param userID   想要邀请的房间内用户
     * @param callback 邀请回调
     */
    public void addCoHost(String userID, ZegoRoomCallback callback) {
        ZegoCustomCommand command = new ZegoCustomCommand();
        command.actionType = ZegoCustomCommand.CustomCommandType.Invitation;
        command.targetUserIDs = Collections.singletonList(userID);
        command.toJson();
        ZegoZIMManager.getInstance().zim.sendPeerMessage(command, userID, new ZIMMessageSendConfig(),
            (message, errorInfo) -> {
                if (errorInfo.code.value() == ZegoRoomErrorCode.SUCCESS) {
                    List<ZegoUserInfo> userInfoList = ZegoRoomManager.getInstance().userService.getUserList();
                    for (ZegoUserInfo zegoUserInfo : userInfoList) {
                        if (Objects.equals(userID, zegoUserInfo.getUserID())) {
                            zegoUserInfo.setHasInvited(true);
                            handler.postDelayed(() -> {
                                zegoUserInfo.setHasInvited(false);
                                if (listener != null) {
                                    listener.onRoomUserInfoUpdate(userInfoList);
                                }
                            }, RESET_INVITED_DELAY_TIME);
                            break;
                        }
                    }
                }

                if (callback != null) {
                    callback.onRoomCallback(errorInfo.code.value());
                }
            });
    }

    /**
     * 回应连麦邀请
     * <p>
     * 描述：这个方法用于接受或者拒绝房主的连麦邀请
     * <p>
     * 调用时机：在加入房间之后
     *
     * @param accept:  传入true或者false来接受或者拒绝邀请
     * @param callback 回应邀请回调
     */
    public void respondCoHostInvitation(boolean accept, String operateUserID, ZegoRoomCallback callback) {
        ZegoCustomCommand command = new ZegoCustomCommand();
        command.actionType = ZegoCustomCommand.CustomCommandType.RespondInvitation;
        command.targetUserIDs = Collections.singletonList(operateUserID);
        command.content = new ZegoCustomCommand.CustomCommandContent(accept);
        command.toJson();
        ZegoZIMManager.getInstance().zim.sendPeerMessage(command, operateUserID, new ZIMMessageSendConfig(),
            new ZIMMessageSentCallback() {
                @Override
                public void onMessageSent(ZIMMessage message, ZIMError errorInfo) {
                    if (callback != null) {
                        callback.onRoomCallback(errorInfo.code.value());
                    }
                }
            });
    }

    /**
     * 申请连麦
     * <p>
     * 描述：这个方法可以用来发送连麦申请给房主
     * <p>
     * 调用时机：在进入房间之后
     *
     * @param callback 发送连麦申请回调
     */
    public void requestToCoHost(ZegoRoomCallback callback) {
        Triple<HashMap<String, String>, String, ZIMRoomAttributesSetConfig> triple
                = ZegoRoomAttributesHelper.getRequestOrCancelToHostParameters(true);
        ZegoRoomAttributesHelper.setRoomAttributes(triple.first, triple.second, triple.third, callback);
    }

    /**
     * 取消连麦申请
     * <p>
     * 描述：这个方法用于参与者取消连麦申请
     * <p>
     * 调用时机：加入房间之后
     *
     * @param callback 取消连麦申请回调
     */
    public void cancelRequestToCoHost(ZegoRoomCallback callback) {
        Triple<HashMap<String, String>, String, ZIMRoomAttributesSetConfig> triple
                = ZegoRoomAttributesHelper.getRequestOrCancelToHostParameters(false);
        ZegoRoomAttributesHelper.setRoomAttributes(triple.first, triple.second, triple.third, callback);
    }

    /**
     * 回应连麦申请
     * <p>
     * 描述：这个方法用于房主回应参与者的连麦申请
     * 当参与者收到连麦邀请时可以调用 takeSeat 方法来连麦
     * <p>
     * 调用时机：进入房间之后
     *
     * @param callback 回应连麦申请回调
     */
    public void respondCoHostRequest(boolean agree, String userID, ZegoRoomCallback callback) {
        Triple<HashMap<String, String>, String, ZIMRoomAttributesSetConfig> triple
                = ZegoRoomAttributesHelper.getRespondCoHostParameters(agree, userID);
        if (triple != null) {
            ZegoRoomAttributesHelper.setRoomAttributes(triple.first, triple.second, triple.third, callback);
        }
    }

    /**
     * 静音连麦用户
     * <p>描述：这个方法可以用来控制连麦用户静音或解除静音
     * 当连麦用户被房主静音，他只能等待房主解除静音猜能说话
     * <p>
     * 调用时机：进入房间之后
     *
     * @param isMuted  决定静音还是解除静音，true: 静音. false: 解除静音
     * @param userID   指想要静音的连麦用户
     * @param callback 经营连麦用户回调
     */
    public void muteUser(boolean isMuted, String userID, ZegoRoomCallback callback) {
        Triple<HashMap<String, String>, String, ZIMRoomAttributesSetConfig> triple
                = ZegoRoomAttributesHelper.getSeatChangeParameters(userID, isMuted, 2);
        if (triple != null) {
            ZegoRoomAttributesHelper.setRoomAttributes(triple.first, triple.second, triple.third, callback);
        }
    }

    /**
     * 麦克风操作
     * <p>
     * 描述：这个方法可以用来开启/关闭麦克风
     * 当麦克风开启时，音频流将会推送到远端的用户，当麦克风关闭时，音频流停止推送
     * <p>
     * 调用时机：在加入房间之后
     *
     * @param open     决定开麦还是关麦，true: 开启 false: 关闭
     * @param callback 开关麦回调
     */
    public void micOperate(boolean open, ZegoRoomCallback callback) {
        Triple<HashMap<String, String>, String, ZIMRoomAttributesSetConfig> triple
                = ZegoRoomAttributesHelper.getSeatChangeParameters(localUserInfo.getUserID(), open, 0);
        if (triple != null) {
            ZegoRoomAttributesHelper.setRoomAttributes(triple.first, triple.second, triple.third, errorCode -> {
                if (callback != null) {
                    callback.onRoomCallback(errorCode);
                }
            });
        }
    }

    /**
     * 相机相关操作
     * <p>
     * 描述：调用该方法来开启关闭相机。当相机开启时，视频流将会推送到远端用户，当相机关闭时，将会停止推送视频流
     * <p>
     * 调用时机：在加入房间之后
     *
     * @param open     决定是否打开相机。 true: 打开  false: 关闭
     * @param callback 开启或关闭相机回调
     */
    public void cameraOperate(boolean open, ZegoRoomCallback callback) {
        Triple<HashMap<String, String>, String, ZIMRoomAttributesSetConfig> triple
                = ZegoRoomAttributesHelper.getSeatChangeParameters(localUserInfo.getUserID(), open, 1);
        if (triple != null) {
            ZegoRoomAttributesHelper.setRoomAttributes(triple.first, triple.second, triple.third, errorCode -> {
                if (callback != null) {
                    callback.onRoomCallback(errorCode);
                }
            });
        }
    }

    /**
     * 上麦
     * <p>
     * 描述：这个方法用来上麦
     * 当这个方法被调用时所有房间用户都会收到通知，麦上用户会变化，并且会拉刚刚上麦用户的流。
     * <p>
     * 调用时机：在加入房间之后
     *
     * @param callback 上麦回调
     */
    public void takeSeat(ZegoRoomCallback callback) {
        Triple<HashMap<String, String>, String, ZIMRoomAttributesSetConfig> triple
                = ZegoRoomAttributesHelper.getTakeOrLeaveSeatParameters(null, true);
        ZegoRoomAttributesHelper.setRoomAttributes(triple.first, triple.second, triple.third, errorCode -> {
            if (errorCode == ZegoRoomErrorCode.SUCCESS) {
                ZegoDeviceService deviceService = ZegoRoomManager.getInstance().deviceService;
                ZegoExpressEngine.getEngine().startPublishingStream(ZegoLiveHelper.getSelfStreamID());
                deviceService.enableCamera(true);
                deviceService.muteMic(false);
            }
            if (callback != null) {
                callback.onRoomCallback(errorCode);
            }
        });
    }

    /**
     * 下麦
     * <p>
     * 描述：这个方法用于下麦
     * 当这个方法被调用时所有房间用户都会收到通知，麦上用户会变化，并且会停止拉刚刚上麦用户的流。
     * <p>
     * 调用时机：在加入房间之后
     *
     * @param callback 下麦回调
     */
    public void leaveSeat(String userID, ZegoRoomCallback callback) {
        Triple<HashMap<String, String>, String, ZIMRoomAttributesSetConfig> triple
                = ZegoRoomAttributesHelper.getTakeOrLeaveSeatParameters(userID, false);
        ZegoRoomAttributesHelper.setRoomAttributes(triple.first, triple.second, triple.third, errorCode -> {
            if (errorCode == ZegoRoomErrorCode.SUCCESS && UserInfoHelper.isUserIDSelf(userID)) {
                ZegoExpressEngine.getEngine().stopPublishingStream();
            }
            if (callback != null) {
                callback.onRoomCallback(errorCode);
            }
        });
    }

    public void setListener(ZegoUserServiceListener listener) {
        this.listener = listener;
    }

    public void onRoomMemberJoined(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
        List<ZegoUserInfo> joinUsers = generateRoomUsers(memberList);
        Iterator<ZegoUserInfo> iterator = joinUsers.iterator();
        while (iterator.hasNext()) {
            ZegoUserInfo next = iterator.next();
            if (!userMap.containsKey(next.getUserID())) {
                userList.add(next); // 避免重复
                userMap.put(next.getUserID(), next);
            } else {
                // 如果重复了就不要通知外部
                iterator.remove();
            }
        }
        if (joinUsers.size() > 0 && listener != null) {
            listener.onRoomUserJoin(joinUsers);
        }
    }

    public void onRoomMemberLeft(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
        List<ZegoUserInfo> leaveUsers = generateRoomUsers(memberList);
        userList.removeAll(leaveUsers);
        for (ZegoUserInfo leaveUser : leaveUsers) {
            userMap.remove(leaveUser.getUserID());
            if (UserInfoHelper.isSelfHost() && UserInfoHelper.isUserIDCoHost(leaveUser.getUserID())) {
                leaveSeat(leaveUser.getUserID(), null);
            }
        }
        if (listener != null) {
            listener.onRoomUserLeave(leaveUsers);
        }
    }

    private List<ZegoUserInfo> generateRoomUsers(List<ZIMUserInfo> memberList) {
        ZegoRoomInfo roomInfo = ZegoRoomManager.getInstance().roomService.roomInfo;

        List<ZegoUserInfo> roomUsers = new ArrayList<>();
        for (ZIMUserInfo userInfo : memberList) {
            ZegoUserInfo roomUser = new ZegoUserInfo();
            roomUser.setUserID(userInfo.userID);
            roomUser.setUserName(userInfo.userName);

            if (userInfo.userID.equals(roomInfo.getHostID())) {
                roomUser.setRole(ZegoRoomUserRole.Host);
            } else {
                roomUser.setRole(ZegoRoomUserRole.Participant);
            }
            roomUsers.add(roomUser);
        }
        return roomUsers;
    }

    public List<ZegoUserInfo> getUserList() {
        return userList;
    }

    public ZegoUserInfo getUserInfo(String userID) {
        return userMap.get(userID);
    }

    public String getUserName(String userID) {
        ZegoUserInfo zegoUserInfo = userMap.get(userID);
        if (zegoUserInfo != null) {
            return zegoUserInfo.getUserName();
        } else {
            return "";
        }
    }

    public void onRoomAttributesUpdated(HashMap<String, String> roomAttributes, OperationCommand command) {
        String myUserID = localUserInfo.getUserID();
        OperationAction action = command.getAction();

        switch (action.getType()) {
            case RequestToCoHost:
                if (UserInfoHelper.isSelfHost()) {
                    if (listener != null) {
                        listener.onReceiveToCoHostRequest(action.getTargetID());
                    }
                }
                break;
            case CancelRequestCoHost:
                if (UserInfoHelper.isSelfHost()) {
                    if (listener != null) {
                        listener.onReceiveCancelToCoHostRequest(action.getTargetID());
                    }
                }
                break;
            case AgreeToCoHost:
                if (Objects.equals(myUserID, action.getTargetID())) {
                    if (listener != null) {
                        listener.onReceiveToCoHostRespond(true);
                    }
                }
                break;
            case DeclineToCoHost:
                if (Objects.equals(myUserID, action.getTargetID())) {
                    if (listener != null) {
                        listener.onReceiveToCoHostRespond(false);
                    }
                }
                break;
        }

        ZegoCoHostSeatModel seat = null;
        for (ZegoCoHostSeatModel model : command.getSeatList()) {
            if (Objects.equals(model.getUserID(), myUserID)) {
                seat = model;
            }
        }

        if (seat == null) {
            return;
        }

        if (!Objects.equals(myUserID, action.getTargetID())) {
            return;
        }

        ZegoDeviceService deviceService = ZegoRoomManager.getInstance().deviceService;
        if (action.getType() == OperationActionType.Mic) {
            deviceService.muteMic(!seat.isMicEnable());
        }

        if (action.getType() == OperationActionType.Camera) {
            deviceService.enableCamera(seat.isCameraEnable());
        }

        if (action.getType() == OperationActionType.Mute) {
            deviceService.muteMic(seat.isMuted());
            micOperate(!seat.isMuted(), null);
        }
    }

    public void onReceivePeerMessage(ZIM zim, ArrayList<ZIMMessage> messageList, String fromUserID) {
        for (ZIMMessage zimMessage : messageList) {
            if (zimMessage.getType() == ZIMMessageType.COMMAND) {
                ZIMCommandMessage zimCustomMessage = (ZIMCommandMessage) zimMessage;
                ZegoCustomCommand command = new ZegoCustomCommand();
                command.fromJson(zimCustomMessage.message);
                if (command.actionType == ZegoCustomCommand.CustomCommandType.Invitation) {
                    if (listener != null) {
                        listener.onReceiveAddCoHostInvitation(zimMessage.getSenderUserID());
                    }
                } else {
                    ZegoCustomCommand.CustomCommandContent content = command.content;
                    if (content == null) {
                        continue;
                    }
                    List<ZegoUserInfo> userInfoList = ZegoRoomManager.getInstance().userService.getUserList();
                    for (ZegoUserInfo zegoUserInfo : userInfoList) {
                        if (Objects.equals(zimMessage.getSenderUserID(), zegoUserInfo.getUserID())) {
                            zegoUserInfo.setHasInvited(false);
                            break;
                        }
                    }

                    if (listener != null) {
                        listener.onReceiveAddCoHostRespond(zimMessage.getSenderUserID(), content.accept);
                    }
                }
            }
        }
    }
}